function [par_out,error_flag] = plot_grid_add_debtox2019(filename)

% Usage: [par_out,error_flag] = plot_grid_add_debtox2019(filename)
% 
% Function to combine MAT files for two chemicals (as generated by the
% parameter-space explorer) to make a MAT file for the mixture, assuming
% damage addition. This function makes a plot of zb x bb and zs x bs from
% multiple samples, to check visually whether they overlap (which would
% imply that damage addition is plausible). Two (or more) chemials can only
% be additive (in the strictest sense) if mb x bb. Therefore, the parameter
% clouds for the individual chemicals are combined by only taking those
% combinations for which mb x bb very close. This function only works
% properly if the MAT file was generated by the BYOM DEBtox2019 package for
% the simple compound models (incl. ERA_special)! Output is also meant for
% the simple compound DEBtox2019 mixture package.
% 
% Note: this is the function for engine, without using the parallel toolbox.
% 
% filename   cell array with strings for the name of MAT files to combine
% 
% Author     : Tjalling Jager
% Date       : September 2023
% Web support: <http://www.debtox.info/byom.html>

%  Copyright (c) 2012-2024, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

global glo2

SETTINGS_OPTIM = setup_settings(1); % load settings from file (use rough settings)

n_s = length(filename); % number of samples to compare
if n_s > 4 % for now, we only allow two files anyway, so this is for the future
    error_str = ('Cannot compare more than 4 samples at the moment');
    errordlg(error_str,'Error no. of sets'); % display a message box with the errors
    error_flag = 1; % signal main script that we ran into errors
    par_out    = [];
    return % simply stop as there is no useful input to work with
end

%% Run through sample to load files and see where 'good' sets are
% For damage addition, the parameters for chemical A and B are not
% independent. For sub-lethal effects, the product of zb x bb must be the
% same. For lethal effects, under SD, the product of zs x bs must be the
% same.

% First, initialise matrices and cell arrays
BNDS  = cell(1,n_s);
COLL  = cell(1,n_s);
NAMES = cell(1,n_s);
BASIC = cell(1,n_s);
ZBBB  = cell(1,n_s);
plot_best = nan(1,n_s);
plot_neg  = nan(1,n_s);
plot_pos  = nan(1,n_s);
plot_bestS = nan(1,n_s);
plot_negS  = nan(1,n_s);
plot_posS  = nan(1,n_s);

[figh,ft] = make_fig(1,1); % create a figure window of correct size
h_pl1 = subplot(1,2,1); % make a sub-plot
hold on

set(h_pl1,'LineWidth',1,ft.name,ft.ticks) % adapt axis formatting
% p = get(h_pl1,'position');
% p([3 4]) = p([3 4])*1.1; % add 10 percent to width and height
% set(h_pl1, 'position', p);
xlabel('data set')
ylabel('product zb x bb')
xlim([0 n_s+0.5])
% modify tick on x-axis to show chemical A and B
xticks(1:n_s)
xticklabels({'A','B','C','D'}) % any extraneous entries will be ignored anyway

h_pl2 = subplot(1,2,2); % make a sub-plot
hold on
set(h_pl2,'LineWidth',1,ft.name,ft.ticks) % adapt axis formatting
% p = get(h_pl2,'position');
% p([3 4]) = p([3 4])*1.1; % add 10 percent to width and height
% set(h_pl2, 'position', p);
xlabel('data set')
ylabel('product zs x bs')
xlim([0 n_s+0.5])
% modify tick on x-axis to show chemical A and B
xticks(1:n_s)
xticklabels({'A','B','C','D'}) % any extraneous entries will be ignored anyway

for i_s = 1:n_s % run through samples
    
    % Load sample <i_s>
    if exist([filename{i_s}],'file') == 2 % check if it exists first.
        load([filename{i_s}],'pmat','coll_all','GLO','names')
        % Load the saved information from the <calc_parspace> run.
    else % otherwise, produce an error (I don't think that is possible anymore)
        error(['There is no confidence set with filename ',filename{i_s},' in your working directory, so run calibration (with correct settings) first.'])
    end
    ind_basic = find(strcmp(names,'kd'))-1; % assume that kd is the first tox parameter!

    % Extract useful information from <pmat>.
    ind_fit  = find(pmat(:,2) == 1); % indices to fitted parameters (vector)
    pmat_lim = pmat(ind_fit,:); % limit <pmat_lim> to fitted parameters
    ind_log  = find(pmat_lim(:,5) == 0); % indices to log-scale parameters in fitted <pmat> (vector)
    
    coll_all(:,ind_log) = 10.^coll_all(:,ind_log); % put back on normal scale, where needed
    names_lim           = names(ind_fit); % only keep names for fitted parameters
    
    % Chi2-criteria and indices to parameter sets to plot for inner and outer rim
    % We work here with the log-likelihood itself, so the chi2 criterion needs to be divided by 2
    chicrit_single = 0.5 * SETTINGS_OPTIM.crit_table(1,1);     % criterion for single-parameter CIs
    chicrit_prop   = 0.5 * SETTINGS_OPTIM.crit_prop(2);        % criterion for upper band of propagation band
    chicrit_prop2  = 0.5 * SETTINGS_OPTIM.crit_prop(1);        % criterion for lower band of propagation band
    
    % Find indices to sets within inner and outer rim
    ind_single = find(coll_all(:,end) < coll_all(1,end) + chicrit_single,1,'last'); % index to last element of sample that is still in inner rim
    
    % collect the results for further processing
    BNDS{i_s}  = pmat(ind_fit,[3 4]); % collect bounds used for original optimisation
    COLL{i_s}  = coll_all;
    NAMES{i_s} = names_lim;

    % put <pmat> on normal scale for log-parameters (this is needed for the
    % non-fitted parameters)
    pmat(pmat(:,5)==0,1) = 10.^(pmat(pmat(:,5)==0,1));

    BASIC{i_s} = pmat(1:ind_basic,1); % collect value for basic parameters
    if any(pmat(1:ind_basic,2) == 1)
        warning('off','backtrace')
        warning(['Basic parameters were fitted in dataset ',filename{i_s},', this is ignored!'])
        warning('on','backtrace')
    end

    % Perhaps also check here that glo.moa and glo.feedb are all the same!
            
    ind_zb     = strcmp(names_lim,'zb'); % find location of <zb> in <pmat_lim>
    ind_bb     = strcmp(names_lim,'bb'); % find location of <bb> in <pmat_lim>
    zbbb_all   = coll_all(:,ind_zb) .* coll_all(:,ind_bb); % product of zb and bb
    zbbb_inner = zbbb_all(1:ind_single,:); % points in the inner rim

    ind_zs     = strcmp(names_lim,'zs'); % find location of <zs> in <pmat_lim>
    ind_bs     = strcmp(names_lim,'bs'); % find location of <bs> in <pmat_lim>
    zsbs_all   = coll_all(:,ind_zs) .* coll_all(:,ind_bs); % product of zs and bs
    zsbs_inner = zsbs_all(1:ind_single,:); % points in the inner rim

    % plot comparison plot as best plus error bar for energy budget
    plot_best(i_s) = zbbb_all(1); % best value
    plot_neg(i_s)  = plot_best(i_s) - min(zbbb_inner); % length of negative error bar
    plot_pos(i_s)  = max(zbbb_inner) - plot_best(i_s); % length of positive error bar
    errorbar(h_pl1,i_s,plot_best(i_s),plot_neg(i_s),plot_pos(i_s),'ko','MarkerFaceColor','y','LineWidth',1)

    % plot comparison plot as best plus error bar for survival
    plot_bestS(i_s) = zsbs_all(1); % best value
    plot_negS(i_s)  = plot_bestS(i_s) - min(zsbs_inner); % length of negative error bar
    plot_posS(i_s)  = max(zsbs_inner) - plot_bestS(i_s); % length of positive error bar
    errorbar(h_pl2,i_s,plot_bestS(i_s),plot_negS(i_s),plot_posS(i_s),'ko','MarkerFaceColor','r','LineWidth',1)

%         % alternatively, plot points in the comparison plot
%         plot(i_s,mwbw_outer,'ko','MarkerFaceColor','w','MarkerEdgeColor','c') % plot all tries to continue with (or outer rim) as cyan open circles)
%         plot(i_s,mwbw_inner,'ko','MarkerFaceColor','g')
%         plot(i_s,mwbw_all(1),'ko','MarkerFaceColor','y')
        
        % collect the results for further processing
        ZBBB{i_s} = zbbb_all;
        ZSBS{i_s} = zsbs_all;
end

drawnow

% Test to see if the CIs overlap ... I think this works for binary
% mixtures, but not for tertiary and quaternary mixtures ...
plot_min = plot_best - plot_neg;
plot_max = plot_pos + plot_best;
ind_max  = plot_max == max(plot_max); % finds highest upper CI of the comparison metric
% now the upper intervals of the other data sets must be above the lower
% interval of the max ...
ind_tst = plot_max ~= max(plot_max); % ones that do not have the highest upper CI
disp(' ')
if any(plot_max(ind_tst) < plot_min(ind_max))
    disp('It seems that the CIs for sub-lethal effects do not overlap, indicating that damage addition is unlikely.')
else
    if n_s == 2
        disp('The CIs for sub-lethal effects overlap, indicating that damage addition is a possibility.')
    else
        disp('Some CIs for sub-lethal effects overlap, but this code cannot determine if they all do, so please check.')
    end
end

% Also do this for survival parameters!
plot_min = plot_bestS - plot_negS;
plot_max = plot_posS + plot_bestS;
ind_max  = plot_max == max(plot_max); % finds highest upper CI of the comparison metric
% now the upper intervals of the other data sets must be above the lower
% interval of the max ...
ind_tst = plot_max ~= max(plot_max); % ones that do not have the highest upper CI
if any(plot_max(ind_tst) < plot_min(ind_max))
    disp('It seems that the CIs for lethal effects do not overlap, indicating that damage addition is unlikely.')
else
    if n_s == 2
        disp('The CIs for lethal effects overlap, indicating that damage addition is a possibility.')
    else
        disp('Some CIs for lethal effects overlap, but this code cannot determine if they all do, so please check.')
    end
end
disp(' ')

%% Create a new <coll_all> matrix for use with mixtures

limit      = 0.02; % factor within which the parameters must be to count as 'the same'
coll_all   = []; % start with empty <coll_all> matrix

% glo2.names = {'hb';'kdA';'kdB';'mw';'bw';'Fs';'WB';'IAB'}; % names of the model parameters for new <coll_all> and <pmat>

ind_kd = find(strcmp(names,'kd'));
glo2.names = cat(1,names(1:ind_kd-1),{'kdA';'kdB'},names(ind_kd+1:end),{'WB';'IAB'}); % names of the model parameters for new <coll_all> and <pmat>

disp('Combining MAT files, this may take a while. Progress will be shown in wait bar.')
f = waitbar(0,'Combining MAT files for DEBtox2019. Please wait.','Name','plot_grid_add_debtox2019.m');

% find indices for all model parameters in names vector for each chemical
ind_kdA = strcmp(NAMES{1},'kd');
ind_kdB = strcmp(NAMES{2},'kd');
ind_zbA = strcmp(NAMES{1},'zb');
ind_bbA = strcmp(NAMES{1},'bb');
ind_zbB = strcmp(NAMES{2},'zb');
ind_bbB = strcmp(NAMES{2},'bb');

ind_zsA = strcmp(NAMES{1},'zs');
ind_bsA = strcmp(NAMES{1},'bs');
% ind_zsB = strcmp(NAMES{2},'zs');
% ind_bsB = strcmp(NAMES{2},'bs');

% Parameters for A will be the basis of the new coll_all. With kdA and kdB
% separate, and a weight factor calculated from sub-lethal effect
% parameters only. This is a bit limited ... but hopefully suffices to
% generate a proper MAT file as starting point for a mixture fit.

coll_all_coll = cell(length(COLL{1}(:,1)),1);
for iA = 1:length(COLL{1}(:,1)) % run through all elements for compound A

    % for each element in A, find where the product zb x bb is similar in B
    ind_ok = find(abs(ZBBB{1}(iA)-ZBBB{2})/ZBBB{1}(iA) < limit);

    coll_add      = nan(length(ind_ok),8); % what we're adding to coll_all for set iA
    coll_add(:,1) = COLL{1}(iA,ind_kdA);     % collect kdA
    coll_add(:,2) = COLL{2}(ind_ok,ind_kdB); % collect kdB
    coll_add(:,3) = COLL{1}(iA,ind_zbA);     % collect zbA
    coll_add(:,4) = COLL{1}(iA,ind_bbA);     % collect bbA
    coll_add(:,5) = COLL{1}(iA,ind_zsA);     % collect zsA
    coll_add(:,6) = COLL{1}(iA,ind_bsA);     % collect bsA

    % weight factor will be the mean of the ratio zbA/zbB and bbB/bbA
    coll_add(:,7) = mean([COLL{1}(iA,ind_zbA)./COLL{2}(ind_ok,ind_zbB), COLL{2}(ind_ok,ind_bbB)./COLL{1}(iA,ind_bbA)],2);
    % min-log-likelihood is sum of that for the individual compounds
    coll_add(:,8) = COLL{1}(iA,end)+COLL{2}(ind_ok,end);

    coll_all_coll{iA} = coll_add;

    waitbar(min(1,iA/length(COLL{1}(:,1))),f); % update waiting bar

end

for iA = 1:length(COLL{1}(:,1)) % run through all elements for compound A
    coll_all = cat(1,coll_all,coll_all_coll{iA}); % add to total coll_all
end

close(f) % close the waiting bar


if isempty(coll_all)
    warning('No proper parameter combinations for addition can be found; no MAT file saved')
    error_flag = 1; % signal main script that we ran into errors
    par_out    = [];
    return % simply stop as there is no useful input to work with
end

coll_all      = sortrows(coll_all,size(coll_all,2));  % sort <coll_all> based on minloglik
chicrit_joint = 0.5 * SETTINGS_OPTIM.crit_table(5,1); % criterion for joint 95% CI or parameters
% Find indices to sets within inner and outer rim.
ind_fin95  = find(coll_all(:,end) < coll_all(1,end) + chicrit_joint,1,'last'); % index to last element of sample that is still in joint CI
ind_prop   = find(coll_all(:,end) < coll_all(1,end) + chicrit_prop,1,'last');  % index to last element of sample that is still in propagation band

coll_all(ind_fin95+1:end,:) = []; % remove the really bad ones

% If inner rim (incl. propagation set) is too large, we can downsample
if ind_prop > 6000
    ind_thin = randperm(ind_prop,ind_prop-5000); % downsample to keep an inner rim approx. 5000 sets (incl. propagation band)
    coll_all(1+ind_thin,:) = []; % remove some elements from outer rim! (but NOT the best value!)
    ind_fin95  = find(coll_all(:,end) < coll_all(1,end) + chicrit_joint,1,'last'); % index to last element of sample that is still in joint CI
    ind_prop   = find(coll_all(:,end) < coll_all(1,end) + chicrit_prop,1,'last');  % index to last element of sample that is still in propagation band
end

% we don't need so many in the outer rim, so we can downsample to decrease
% size of the mat file produced
thin_sz  = ind_fin95-ind_prop;
ind_thin = randperm(thin_sz,max(1,thin_sz-ind_prop*3)); % downsample to keep an outer rim approx. 3x the propagation set
coll_all(ind_prop+ind_thin,:) = []; % remove some elements from outer rim!

ind_fin95  = find(coll_all(:,end) < coll_all(1,end) + chicrit_joint,1,'last');  % index to last element of sample that is still in joint CI
ind_prop   = find(coll_all(:,end) < coll_all(1,end) + chicrit_prop,1,'last');   % index to last element of sample that is still in upper edge propagation band
ind_prop2  = find(coll_all(:,end) < coll_all(1,end) + chicrit_prop2,1,'last');  % index to last element of sample that is still in lower edge propagation band
ind_single = find(coll_all(:,end) < coll_all(1,end) + chicrit_single,1,'last'); % index to last element of sample that is still in inner rim

%% Create new <pmat> for mixture

pmat_rem = pmat;
pmat     = zeros(length(glo2.names),5); % create a new <pmat> for the mixture addition case

pmat(1:ind_basic,1) = mean([BASIC{1} BASIC{2}],2); % for basic parameters, take means from the data sets!
pmat(1:ind_basic,3:5) = pmat_rem(1:ind_basic,3:5); % this uses the columns 3:5 from the LAST set!

% find indices for all model parameters in new names vector for each chemical
ind_kdA2 = find(strcmp(glo2.names,'kdA'));
ind_kdB2 = find(strcmp(glo2.names,'kdB'));
ind_zb  = strcmp(glo2.names,'zb');
ind_bb  = strcmp(glo2.names,'bb');
ind_zs  = strcmp(glo2.names,'zs');
ind_bs  = strcmp(glo2.names,'bs');
ind_WB  = strcmp(glo2.names,'WB');

% fill <pmat> with information from new <coll_all> for parameters common to SD and IT
pmat(ind_kdA2,:) = [coll_all(1,1) 1 min(coll_all(:,1)) max(coll_all(:,1)) 0]; % kdA
pmat(ind_kdB2,:) = [coll_all(1,2) 1 min(coll_all(:,2)) max(coll_all(:,2)) 0]; % kdB
pmat(ind_zb,:) = [coll_all(1,3) 1 min(coll_all(:,3)) max(coll_all(:,3)) 0]; % zb
pmat(ind_bb,:) = [coll_all(1,4) 1 min(coll_all(:,4)) max(coll_all(:,4)) 0]; % bb
pmat(ind_zs,:) = [coll_all(1,5) 1 min(coll_all(:,5)) max(coll_all(:,5)) 0]; % zs
pmat(ind_bs,:) = [coll_all(1,6) 1 min(coll_all(:,6)) max(coll_all(:,6)) 0]; % bs
pmat(ind_WB,:) = [coll_all(1,7) 1 min(coll_all(:,7)) max(coll_all(:,7)) 0]; % WB
pmat(find(ind_WB==1)+1,:) = [0     0 -100  100 1]; % IAB

% Make the bounds a bit wider, but not wider than the original bounds in the saved <pmat>
bnds_extra = 1.5; % what to divide/multiply min-max bounds with to make them a bit wider than sample

pmat([ind_kdA2 ind_kdB2],3) = max(pmat([ind_kdA2 ind_kdB2],3)/bnds_extra,[BNDS{1}(ind_kdA,1);BNDS{2}(ind_kdB,1)]); % min bounds for kdA and kdB
pmat([ind_kdA2 ind_kdB2],4) = min(pmat([ind_kdA2 ind_kdB2],4)*bnds_extra,[BNDS{1}(ind_kdA,2);BNDS{2}(ind_kdB,2)]); % max bounds for kdA and kdB
pmat(ind_zb,3) = max(pmat(ind_zb,3)/bnds_extra,BNDS{1}(ind_zbA,1)); % min bounds for zb
pmat(ind_zb,4) = min(pmat(ind_zb,4)*bnds_extra,BNDS{1}(ind_zbA,2)); % max bounds for zb
pmat(ind_bb,3) = max(pmat(ind_bb,3)/bnds_extra,BNDS{1}(ind_bbA,1)); % min bounds for bb
pmat(ind_bb,4) = min(pmat(ind_bb,4)*bnds_extra,BNDS{1}(ind_bbA,2)); % max bounds for bb
pmat(ind_zs,3) = max(pmat(ind_zs,3)/bnds_extra,BNDS{1}(ind_zsA,1)); % min bounds for zs
pmat(ind_zs,4) = min(pmat(ind_zs,4)*bnds_extra,BNDS{1}(ind_zsA,2)); % max bounds for zs
pmat(ind_bs,3) = max(pmat(ind_bs,3)/bnds_extra,BNDS{1}(ind_bsA,1)); % min bounds for bs
pmat(ind_bs,4) = min(pmat(ind_bs,4)*bnds_extra,BNDS{1}(ind_bsA,2)); % max bounds for bs
pmat(ind_WB,3) = pmat(ind_WB,3)/bnds_extra; % min bounds for WB
pmat(ind_WB,4) = pmat(ind_WB,4)*bnds_extra; % max bounds for WB

ind_fit = find(pmat(:,2) == 1); % indices to fitted parameters (vector)

% bounds that are smaller than a factor 10 can be fitted on normal scale
chk_bnds = pmat(:,2) ==1 & (pmat(:,4)./pmat(:,3))<10; % which elements are smaller than 10?
pmat(chk_bnds,5) = 1; % put them to normal scale

%% Display, plot and final things

% display par in a formatted way so they can be directly copied into the code of the main script
par_out = packunpack(2,[],pmat);
disp(' ')
disp('  Damage addition for DEBtox2019')
disp('You can copy-paste following lines into script for mixture predictions')
disp('if you like to run the parameter-space explorer on the mixture data.')
print_par(par_out) % this prints out the optimised parameter values in a formatted manner

pmat_lim = pmat(ind_fit,:); % limited parameter matrix for fitted parameters
ind_log  = find(pmat_lim(:,5) == 0); % indices to log-scale parameters in fitted <pmat> (vector)

% improvise a <pmat_print>, though without any CIs, so it can be saved
pmat_print      = zeros(length(ind_fit),8);
pmat_print(:,1) = pmat_lim(:,1); % copy best-fit values
% add CIs for the model parameters, estimated as the edges of the
% propagation set; this is a good approximation
pmat_print(:,2) = (min(coll_all(1:ind_prop,1:end-1),[],1))'; 
pmat_print(:,3) = (max(coll_all(1:ind_prop,1:end-1),[],1))';

disp(' ')
disp('=================================================================================')
disp('Results of the parameter-space combination for mixture predictions')
disp('=================================================================================')
disp('Damage addition for DEBtox2019')
disp(['   Chemical A is taken from: ',filename{1}])
disp(['   Chemical B is taken from: ',filename{2}])
disp(['   Sample: ',num2str(ind_fin95),' sets in joint CI and ',num2str(ind_single),' in inner CI.'])
disp(['   Propagation set: ',num2str(1+ind_prop - ind_prop2),' sets will be used for error propagation.'])

FVAL = coll_all(1,end);
fprintf('   Minus log-likelihood has reached the value %#1.2f (AIC=%#1.2f). \n',FVAL,2*length(ind_fit)+2*FVAL)

disp('Approx. best estimates and 95% CIs on fitted parameters')
disp_pmat(pmat,pmat_print); % call dedicated function for printing the <pmat>
disp('Confidence intervals are estimated from the sample (inner rim, incl. propagation band)')
disp('so they will generally slightly exaggerate the true intervals.')

% put <pmat> on log-scale for log-parameters (this is how BYOM uses it)
% pmat(pmat(:,2)==1 & pmat(:,5)==0,1) = log10(pmat(pmat(:,2)==1 & pmat(:,5)==0,1));
pmat(pmat(:,5)==0,1) = log10(pmat(pmat(:,5)==0,1)); % for ALL parameters

% put <coll_all> on log-scale for log-parameters
coll_all(:,ind_log) = log10(coll_all(:,ind_log));

coll_prof_pruned = cell(1,length(ind_fit));
for i_p = 1:length(ind_fit) % trick to make sure profiles are plotted (but without refinement red line)
    coll_prof_pruned{i_p} = [NaN NaN];
end

% And make a plot of the final results as parameter-space plot
figh = plot_grid(pmat,[],coll_all,coll_prof_pruned,[],SETTINGS_OPTIM); 

% user can give a filename for saving, but I try to provide a good default
% filenm = ['ADD_',filename{1}(13:end-26),'+',filename{2}(13:end)];
filenm = ['ADD_',filename{1}(13:end-26),'+',filename{2}(13:end-26),'_PS.mat'];

% % Instead of the line above, feel free to uncomment the code below to use
% % the Matlab GUI for entering a filename for saving.
% filenm = uiputfile(['ADD_',filename{1}(11:end-12),'+',filename{2}(11:end)],'Save new project file for mixture of chemical A and B'); % use Matlab GUI to select name for saving MAT file
% if numel(filenm) == 1 && filenm(1) == 0 % if cancel is pressed ...
%     error_flag = 1; % signal main script that we ran into errors
%     return % simply stop as the user does not want to save
% end

saveplt = 0; % for now, don't save (since this function is not normally called from a standard byom script, the glo.saveplot will not be defined)
figure(figh)  % use existing handle and make sure that the multiplot is the current plot
% Add a title to the plot.
axes('Position',[0 0 1 1],'Xlim',[0 1],'Ylim',[0 1],'Box','off','Visible','off','Units','normalized', 'clipping','off');
text(0.5, 1,['File: ',filenm,' (',date,')'],'Interpreter','none','HorizontalAlignment','center','VerticalAlignment', 'top');
if saveplt > 0
    save_plot(figh,['parspace_addmixpred_',glo.basenm]) % save parspace plot in output folder
end
snapnow % for publishing Matlab scripts: make plot appear in the context of the screen output

% save new mat file
par   = par_out;
names = glo2.names ;
save(filenm,'par','pmat','coll_all','pmat_print','coll_prof_pruned','names','GLO')
