function [par_out,pmat_print] = automatic_runs_basic(fit_tox,par,ind_tox,skip_sg,opt_optim,opt_plot,varargin)

% This is a helpful function to run a TKTD model analysis in parts, based
% on the setting in fit_tox. We can fit the basic (non-tox) parameters on
% the control, or the tox parameters only to the complete data set. Note
% that this code expects, by default, the control to have identifier zero.
% However, it is possible to have fit_tox with more than one element: if
% the first is zero, the rest is taken to be the identifiers of the
% treatments that are to be used as controls!
% 
% In general, this function will be used in conjunction with the
% parameter-space explorer (opt_optim.type = 4), at least for fitting the
% toxicity data, but most of it will also work with other optimisation
% routines.
% 
% At this moment, this code is for GUTS, GUTS-immobility and DEBtox2019
% analyses only. I may adapt it in the future for other analyses as well.
% Note that GUTS analysis is triggered by entering ind_tox=-1, and
% GUTS-immobility by ind_tox=-2. This is possible since there is only 1
% basic parameter (hb).
% 
% Select whether to fit the control treatment (0), the toxicity treatments
% (1; the control is shown as well), or both together (2). A good strategy
% is to fit the basic parameters to the control data first (fit_tox=0),
% copy the best values into the parameter matrix above, and then fit the
% toxicity parameter to the complete data set (fit_tox=1). The code below
% automatically keeps the parameters fixed that need to be fixed.
% 
% Author     : Tjalling Jager 
% Date       : February 2024
% Web support: http://www.debtox.info/byom.html

%  Copyright (c) 2012-2024, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

global X0mat glo glo2 DATA

pmat_print = []; % this output is normally returned as empty, but when parspace is used, it contains the CIs

% first, a quick check whether there could be problems with the input
if size(DATA,1) > 1
    if isfield(glo,'names_sep') && ~isempty(glo.names_sep) && fit_tox(1) == 0
        error('Function automatic_run_basic cannot (yet) work properly with basic parameters differing per data set!')
    end
end

if length(fit_tox) > 1 % if fit_tox has more than 1 element ...
    ctrl_ids = fit_tox(2:end); % assume the last ones are the identifiers for controls
    fit_tox = fit_tox(1);
else
    ctrl_ids = 0; % if not provided, assume zero
    if isfield(glo,'int_scen') && ~isempty(glo.int_scen) && ismember(0.1,glo.int_scen)
        if fit_tox(1) == 0
            warning('Note that automatic_run_basic can also include solvent controls with ID=0.1! (use fit_tox = [0 0 0.1])')
        end
    end
end

% To ensure backwards compatability (since I added the MoA 'hazards to repro' again)
if isfield(glo,'moa') && length(glo.moa) == 4
    glo.moa(5) = 0; % make sure the length of MOA is 5
end

% remember the globals as we will change them
X0mat_rem = X0mat;
glo2_rem  = glo2;
DATA_rem  = DATA;

switch fit_tox
    case 0 % for fitting the control response only ...
        % This piece of code selects only the treatment with identifier 0
        % (zero), or the ones specified in fit_tox, and turns fitting off
        % for the toxicity parameters. This does not work with parameters
        % that differ between data sets!

        if par.hb(2) == 0
            disp('Background hazard is set to non-fitted, so we can stop immediately!')
            par_out = par;
            return
        end

        X0mat = X0mat(:,ismember(X0mat(1,:),ctrl_ids)); % only keep controls (ids specified as control)
        pmat  = packunpack(1,par,[]); % turn parameter structure into a matrix
        
        switch ind_tox
            case -1 % this signals that we have a GUTS analysis
                pmat(:,2) = 0; % don't fit ANY parameters now
                par       = packunpack(2,[],pmat); % turn parameter matrix back into a structure
                par.hb(2) = 1; % but DO fit the background hazard rate
            case -2 % this signals that we have a GUTS-immobility analysis
                % Specifically for the immobility package: remove all data other
                % than those for the dead animals. However, startgrid_guts will
                % only use the time vector of the first one for its derivation of
                % search ranges.
                for i_sets = 1:size(DATA,1)
                    DATA(i_sets,[glo.locC glo.locD glo.loc_a glo.loc_i glo.loc_id]) = {0}; % remove all but the survival data from the complete data set
                    % Note: this also works in case there are multiple data sets per state.
                end
                opt_plot.statsup = [glo.locC glo.locD glo.loc_a glo.loc_i glo.loc_id]; % vector with states to suppress in plotting fits
                pmat(:,2) = 0; % don't fit ANY parameters now
                par       = packunpack(2,[],pmat); % turn parameter matrix back into a structure
                par.hb(2) = 1; % but DO fit the background hazard rate
            otherwise % we have a DEBtox analysis
                pmat(ind_tox:end,2) = 0; % don't fit the toxicity parameters now
                par   = packunpack(2,[],pmat); % turn parameter matrix back into a structure
        end
        
        glo2.ctot = ctrl_ids; % make sure that the total concentration vector to analyse only has the ids for the control
        % No need to update WRAP, since packunpack only needs names
        
        % optimise and plot (fitted parameters in par_out)
        par_out = calc_optim(par,opt_optim); % start the optimisation
        calc_and_plot(par_out,opt_plot); % calculate model lines and plot them

        print_par(par_out,-2) % this prints out the optimised parameter values in a
        % formatted way so they can be directly copied into the code above; the
        % -2 means that only fitted parameters are displayed
        
        if opt_optim.type ~= 4 && ~isempty(varargin)
            opt_prof = varargin{1};
            calc_proflik(par_out,'all',opt_prof,opt_optim);  % calculate a profile
        end
        
        % =================================================================
        if isfield(par_out,'EHb') % then we're running stdDEB!
            % this needs to be tested for multiple data sets!
            stdDEB_start = cell(1,1:size(DATA,1));
            for i_set = 1:1:size(DATA,1) % run through data sets
                [~,~,~,~,startvals] = call_deri(glo.t,par_out,X0mat,glo);
                stdDEB_start{i_set} = startvals;
            end
            glo.stdDEB_start = stdDEB_start; % add starting values to the global glo with the values per data set
            % NOTE: this should be a huge time saver as these values will
            % not change when running the tox data. This assumes that we
            % don't fit any of the basic parameters along the tox
            % parameters!
        end
        % =================================================================

    case 1 % for fitting tox parameters only ...
        % This piece of code turns fitting off for the basic parameters so only
        % the toxicity parameters are fitted.
        
        switch ind_tox
            case -1 % this signals that we have a GUTS analysis
                par.hb(2) = 0; % turn fitting for the background hazard off
                if opt_optim.type == 4 && skip_sg ~= 1 % for the parspace explorer ...
                    par = startgrid_guts(par); % replace par by estimates based on data set (as used by openGUTS)
                end
            case -2 % this signals that we have a GUTS-immobility analysis
                par.hb(2) = 0; % turn fitting for the background hazard off
                if opt_optim.type == 4 && skip_sg ~= 1 % for the parspace explorer ...
                    par = startgrid_guts_immob(par); % replace par by estimates based on data set (as used by openGUTS)
                end
            otherwise % we have a DEBtox analysis
                pmat = packunpack(1,par,[]); % turn output parameter structure into a matrix
                pmat(1:ind_tox-1,2) = 0; % turn fitting for the basic parameters off in this vector
                par  = packunpack(2,[],pmat); % turn parameter matrix into a structure
                if opt_optim.type == 4 && skip_sg ~= 1 % for the parspace explorer ...
                    par = startgrid_debtox(par); % create search ranges for the fitted tox parameters
                end
        end
                
        % optimise and plot (fitted parameters in par_out)
        [par_out,~,~,pmat_print] = calc_optim(par,opt_optim); % start the optimisation
        if ~isempty(opt_plot)
            calc_and_plot(par_out,opt_plot); % calculate model lines and plot them
        end
        
    case 2 % for fitting all parameters
        if opt_optim.type == 4 && skip_sg ~= 1 % for the parspace explorer ...
            switch ind_tox
                case -1 % this signals that we have a GUTS analysis
                    par = startgrid_guts(par); % replace par by estimates based on data set (as used by openGUTS)
                case -2 % this signals that we have a GUTS-immobility analysis
                    par = startgrid_guts_immob(par); % replace par by estimates based on data set (as used by openGUTS)
                otherwise % we have a DEBtox analysis
                    par = startgrid_debtox(par); % create search ranges for the fitted tox parameters
            end
        end  
        
        % optimise and plot (fitted parameters in par_out)
        [par_out,~,~,pmat_print] = calc_optim(par,opt_optim); % start the optimisation
        if ~isempty(opt_plot)
            calc_and_plot(par_out,opt_plot); % calculate model lines and plot them
        end
end

% return the globals to their original values
X0mat = X0mat_rem;
glo2  = glo2_rem;
DATA  = DATA_rem;
drawnow
